"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __generator = (this && this.__generator) || function (thisArg, body) {
    var _ = { label: 0, sent: function() { if (t[0] & 1) throw t[1]; return t[1]; }, trys: [], ops: [] }, f, y, t, g;
    return g = { next: verb(0), "throw": verb(1), "return": verb(2) }, typeof Symbol === "function" && (g[Symbol.iterator] = function() { return this; }), g;
    function verb(n) { return function (v) { return step([n, v]); }; }
    function step(op) {
        if (f) throw new TypeError("Generator is already executing.");
        while (_) try {
            if (f = 1, y && (t = op[0] & 2 ? y["return"] : op[0] ? y["throw"] || ((t = y["return"]) && t.call(y), 0) : y.next) && !(t = t.call(y, op[1])).done) return t;
            if (y = 0, t) op = [op[0] & 2, t.value];
            switch (op[0]) {
                case 0: case 1: t = op; break;
                case 4: _.label++; return { value: op[1], done: false };
                case 5: _.label++; y = op[1]; op = [0]; continue;
                case 7: op = _.ops.pop(); _.trys.pop(); continue;
                default:
                    if (!(t = _.trys, t = t.length > 0 && t[t.length - 1]) && (op[0] === 6 || op[0] === 2)) { _ = 0; continue; }
                    if (op[0] === 3 && (!t || (op[1] > t[0] && op[1] < t[3]))) { _.label = op[1]; break; }
                    if (op[0] === 6 && _.label < t[1]) { _.label = t[1]; t = op; break; }
                    if (t && _.label < t[2]) { _.label = t[2]; _.ops.push(op); break; }
                    if (t[2]) _.ops.pop();
                    _.trys.pop(); continue;
            }
            op = body.call(thisArg, _);
        } catch (e) { op = [6, e]; y = 0; } finally { f = t = 0; }
        if (op[0] & 5) throw op[1]; return { value: op[0] ? op[1] : void 0, done: true };
    }
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.synchronize = exports.connectWallet = exports.disconnectWallet = exports.subscribeToAccountAndChainChanged = void 0;
var local_storage_1 = require("./local-storage");
function subscribeToAccountAndChainChanged(_a) {
    var dispatch = _a.dispatch, providerConnectors = _a.providerConnectors, connectedKeys = _a.connectedKeys;
    var unsubscribes = connectedKeys.map(function (providerKey) {
        var connector = providerConnectors[providerKey];
        if (!connector) {
            throw new Error("Connector not found while subscribing! Please file an issue");
        }
        var onAccountsChanged = function (accounts) {
            dispatch({
                type: "accountChanged",
                payload: { accounts: accounts, providerKey: providerKey },
            });
        };
        var onChainChanged = function (chainId) {
            dispatch({
                type: "chainChanged",
                payload: { chainId: chainId, providerKey: providerKey },
            });
        };
        var accountsChangedUnsubcribe = connector.subscribeToAccountsChanged(onAccountsChanged);
        var chainChangedUnsubscribe = connector.subscribeToChainChanged(onChainChanged);
        return function () {
            accountsChangedUnsubcribe();
            chainChangedUnsubscribe();
        };
    });
    return function () { return unsubscribes.forEach(function (unsubscribe) { return unsubscribe(); }); };
}
exports.subscribeToAccountAndChainChanged = subscribeToAccountAndChainChanged;
function disconnectWallet(_a) {
    var dispatch = _a.dispatch, providerConnectors = _a.providerConnectors, providerKey = _a.providerKey;
    return __awaiter(this, void 0, void 0, function () {
        var connector;
        return __generator(this, function (_b) {
            switch (_b.label) {
                case 0:
                    connector = providerConnectors[providerKey];
                    if (!connector) {
                        throw new Error("Connector not found while disconnecting! Please file an issue");
                    }
                    return [4 /*yield*/, connector.disconnect()];
                case 1:
                    _b.sent();
                    local_storage_1.LocalStorageHelper.removeKey(providerKey);
                    dispatch({ type: "disconnected", payload: { providerKey: providerKey } });
                    return [2 /*return*/];
            }
        });
    });
}
exports.disconnectWallet = disconnectWallet;
function connectWallet(_a) {
    var dispatch = _a.dispatch, providerConnectors = _a.providerConnectors, providerKey = _a.providerKey;
    return __awaiter(this, void 0, void 0, function () {
        var connector, _b, accounts, chainId;
        return __generator(this, function (_c) {
            switch (_c.label) {
                case 0:
                    connector = providerConnectors[providerKey];
                    if (!connector) {
                        throw new Error("Connector not found while connecting! Please file an issue");
                    }
                    return [4 /*yield*/, connector.connect()];
                case 1:
                    _b = _c.sent(), accounts = _b.accounts, chainId = _b.chainId;
                    local_storage_1.LocalStorageHelper.addKey(providerKey);
                    dispatch({
                        type: "connected",
                        payload: {
                            accounts: accounts,
                            chainId: chainId,
                            providerKey: providerKey,
                        },
                    });
                    return [2 /*return*/];
            }
        });
    });
}
exports.connectWallet = connectWallet;
function synchronize(_a) {
    var dispatch = _a.dispatch, providerConnectors = _a.providerConnectors;
    return __awaiter(this, void 0, void 0, function () {
        var rawProviderKeys, providerKeys;
        var _this = this;
        return __generator(this, function (_b) {
            rawProviderKeys = local_storage_1.LocalStorageHelper.getKeys();
            providerKeys = [];
            rawProviderKeys.forEach(function (rawProviderKey) {
                if (!(rawProviderKey in providerConnectors)) {
                    local_storage_1.LocalStorageHelper.removeKey(rawProviderKey);
                    return;
                }
                providerKeys.push(rawProviderKey);
            });
            providerKeys.forEach(function (providerKey) { return __awaiter(_this, void 0, void 0, function () {
                var connector, successfullConnection, err_1;
                return __generator(this, function (_a) {
                    switch (_a.label) {
                        case 0:
                            if (!(providerKey in providerConnectors)) {
                                local_storage_1.LocalStorageHelper.removeKey(providerKey);
                                return [2 /*return*/];
                            }
                            connector = providerConnectors[providerKey];
                            _a.label = 1;
                        case 1:
                            _a.trys.push([1, 3, , 4]);
                            return [4 /*yield*/, connector.synchronize()];
                        case 2:
                            successfullConnection = _a.sent();
                            if (!successfullConnection) {
                                local_storage_1.LocalStorageHelper.removeKey(providerKey);
                                return [2 /*return*/];
                            }
                            dispatch({
                                type: "connected",
                                payload: {
                                    accounts: successfullConnection.accounts,
                                    chainId: successfullConnection.chainId,
                                    providerKey: providerKey,
                                },
                            });
                            return [3 /*break*/, 4];
                        case 3:
                            err_1 = _a.sent();
                            local_storage_1.LocalStorageHelper.removeKey(providerKey);
                            console.warn("Unable to synchronize ".concat(providerKey, ". Got error: ").concat(err_1));
                            return [3 /*break*/, 4];
                        case 4: return [2 /*return*/];
                    }
                });
            }); });
            return [2 /*return*/];
        });
    });
}
exports.synchronize = synchronize;
